#region Using declarations
using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// TD Wave UP
    /// </summary>
    [Description("TD Wave UP")]
    public class IGTDWaveUP : Indicator
    {
//        #region Variables

		#region Filter
		private bool _boolElliottOscillatorFilterON = true;
		private bool _boolIsWaveInTheDirectionOfTheTrend = true;
		private int	_intElliottOscillatorFilterMultiplier = 8;
		private int	_intFast = 5;
		private int	_intLookBackBars = 0;
		private int	_intSlow = 34;
		private double _doubleElliot = 0;
		private double _doubleElliotLookBackBars = 0;
		#endregion
		
//        #region Linear Regression
		private bool _boolLinearRegressionON = true;
		private Color _colorLinearRegressionLines = Color.Blue;
		private Color _colorLinearRegressionBackground = Color.Transparent;
		private DashStyle _dsLinearRegressionMiddleLine = DashStyle.Dash;
		private DashStyle _dsLinearRegressionChannelLines = DashStyle.Dash;
        private double _doubleStandardDeviation = 2.000;
		private int _intLinearRegressionMiddleLineWidth = 2;	
		private int _intLinearRegressionChannelLinesWidth = 1;	
//        #endregion
		
		#region Projection Lines
		// Wave 3
		private bool _boolPlotWave3PriceProjection = true;
		private Color _colorWave3PriceProjectionLines = Color.Blue;
		private Color _colorWave3PriceProjectionText = Color.Blue;
		private DashStyle _dsWave3PriceProjectionLines = DashStyle.Dash;
        private double _doubleWave3Factor1 = 1.618;
        private double _doubleWave3Factor2 = 2.618;
        private double _doubleWave3Factor3 = 4.237;
		private int _intWave3PriceProjectionFontSize = 13;
		private int _intWave3PriceProjectionLinesWidth = 3;	
		
		// Wave 5
		private bool _boolPlotWave5PriceProjection = true;
		private Color _colorWave5PriceProjectionLines = Color.Blue;
		private Color _colorWave5PriceProjectionText = Color.Blue;
		private DashStyle _dsWave5PriceProjectionLines = DashStyle.Dash;
        private double _doubleWave5Factor = 1.618;
        private double _doubleWave5Factor2 = 2.618;
        private double _doubleWave5Factor3 = 4.237;
        private int _intWave5PriceProjectionFontSize = 13;
		private int _intWave5PriceProjectionLinesWidth = 3;

        // Wave C
        private bool _boolPlotWaveCPriceProjection = true;
        private Color _colorWaveCPriceProjectionLines = Color.Blue;
        private Color _colorWaveCPriceProjectionText = Color.Blue;
        private DashStyle _dsWaveCPriceProjectionLines = DashStyle.Dash;
        private double _doubleWaveCfactor = 1.618;
        private int _intWaveCPriceProjectionFontSize = 13;
        private int _intWaveCPriceProjectionLinesWidth = 3;		
		#endregion
		
		#region Retracement Lines
		private bool _boolPlotWave2RetracementLines = true;
		private bool _boolPlotWave4RetracementLines = true;

		private Color[] _colorRetracementLine = {Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green};
		private Color[] _colorRetracementText = {Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green};
		private DashStyle[] _dsRetracementLine = {DashStyle.Dash, DashStyle.Dash, DashStyle.Dash, DashStyle.Dash, DashStyle.Dash, DashStyle.Solid, DashStyle.Dash};
		private double[] _doubleRetracementLevels = { 0.00, 0.236, 0.382, 0.50, 0.618, 0.764, 1.000 };
		private FontStyle _fsRetracementText = FontStyle.Bold;
		private int[] _intRetracementFontSize = {10, 10, 10, 13, 13, 13, 10};
		private int[] _intRetracementLineWidth = {1, 1, 1, 3, 3, 5, 1};
		#endregion
		
		#region Wave Lines
		// arrays
		private ArrayList _alTDWaveHelper;
		private ArrayList _alTDWaveRequirements;
		
		// wave line
		private bool[] _boolPlotWaveLine = {true, true, true, true, true, true, true, true};
		private Color[] _colorWaveLine = {Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green};
		private DashStyle[] _dsWaveLine = {DashStyle.Dash, DashStyle.Dash, DashStyle.Solid, DashStyle.Dash, DashStyle.Solid, DashStyle.Dash, DashStyle.Dash, DashStyle.Solid};
		private int _intTDMult = 1;
		private int[] _intTDWave = {21, 13, 8, 21, 13, 34, 13, 8, 21};
		private int[] _intWaveLineWidth = {2, 2, 5, 2, 5, 2, 2, 5};

		// wave text
		private Color[] _colorWaveText = {Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green, Color.Green};
		private int[] _intWaveTextFontSize = {13, 13, 13, 13, 13, 13, 13, 13, 13};
		private FontStyle[] _fsWaveText = {FontStyle.Bold, FontStyle.Bold, FontStyle.Bold, FontStyle.Bold, FontStyle.Bold, FontStyle.Bold, FontStyle.Bold, FontStyle.Bold, FontStyle.Bold};
		private string[] _strWaveText = {"Start(W1 low)", "W1", "W2", "W3", "W4", "W5", "WA", "WB", "WC(end)"};
		
		// other		
		private double _doubleMult = 1.618; // Multiplier to be applied to absolute difference of above calculations
		private double _doublePointOffset = 0; 
		private double _doublePrice = 0.0;
		private double _doubleWaveWave1Low = 0.0;
		private double _doubleWaveWave3PriceProjection = 0.0;
		
		private int _intCurrentWave = 0;
		
		private string _strWave2RetracementLinesTag = string.Empty;
		private string _strWave3PriceProjectionTag = string.Empty;
		private string _strWave4RetracementLinesTag = string.Empty;
		private string _strWave5PriceProjectionTag = string.Empty;
		
		private TDWaveHelper _CurrentWaveHelper;
		private TDWaveRequirements _NextWaveRequirements;
		private int _intWaveType = -1;
		private TDWaveType _WaveType;
        #endregion
		
//        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= false;
			
			this._alTDWaveHelper = new ArrayList();
			this._alTDWaveRequirements = new ArrayList();
			if (this.WaveType == TDWaveType.DownWave)
			{
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[0] * _intTDMult, _intTDWave[1] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[0],  this._boolPlotWaveLine[0], this._colorWaveLine[0], this._dsWaveLine[0], this._intWaveLineWidth[0])); // start
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[1] * _intTDMult, _intTDWave[2] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[1],  this._boolPlotWaveLine[1], this._colorWaveLine[1], this._dsWaveLine[1], this._intWaveLineWidth[1])); // wave 1
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[2] * _intTDMult, _intTDWave[3] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[2],  this._boolPlotWaveLine[2], this._colorWaveLine[2], this._dsWaveLine[2], this._intWaveLineWidth[2])); // wave 2
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[3] * _intTDMult, _intTDWave[4] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[3],  this._boolPlotWaveLine[3], this._colorWaveLine[3], this._dsWaveLine[3], this._intWaveLineWidth[3])); // wave 3
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[4] * _intTDMult, _intTDWave[5] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[4],  this._boolPlotWaveLine[4], this._colorWaveLine[4], this._dsWaveLine[4], this._intWaveLineWidth[4])); // wave 4
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[5] * _intTDMult, _intTDWave[6] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[5],  this._boolPlotWaveLine[5], this._colorWaveLine[5], this._dsWaveLine[5], this._intWaveLineWidth[5])); // wave 5
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[6] * _intTDMult, _intTDWave[7] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[6],  this._boolPlotWaveLine[6], this._colorWaveLine[6], this._dsWaveLine[6], this._intWaveLineWidth[6])); // wave a
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[7] * _intTDMult, _intTDWave[8] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[7],  this._boolPlotWaveLine[7], this._colorWaveLine[7], this._dsWaveLine[7], this._intWaveLineWidth[7])); // wave b
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[8] * _intTDMult, 00,                         TDDirection.LookForHighs, this._strWaveText[8],  true, Color.Transparent, DashStyle.Solid, 5)); // wave c completed when closes above low close of wave a
			}
			else
			{
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[0] * _intTDMult, _intTDWave[1] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[0],  this._boolPlotWaveLine[0], this._colorWaveLine[0], this._dsWaveLine[0], this._intWaveLineWidth[0])); // start
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[1] * _intTDMult, _intTDWave[2] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[1],  this._boolPlotWaveLine[1], this._colorWaveLine[1], this._dsWaveLine[1], this._intWaveLineWidth[1])); // wave 1
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[2] * _intTDMult, _intTDWave[3] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[2],  this._boolPlotWaveLine[2], this._colorWaveLine[2], this._dsWaveLine[2], this._intWaveLineWidth[2])); // wave 2
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[3] * _intTDMult, _intTDWave[4] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[3],  this._boolPlotWaveLine[3], this._colorWaveLine[3], this._dsWaveLine[3], this._intWaveLineWidth[3])); // wave 3
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[4] * _intTDMult, _intTDWave[5] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[4],  this._boolPlotWaveLine[4], this._colorWaveLine[4], this._dsWaveLine[4], this._intWaveLineWidth[4])); // wave 4
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[5] * _intTDMult, _intTDWave[6] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[5],  this._boolPlotWaveLine[5], this._colorWaveLine[5], this._dsWaveLine[5], this._intWaveLineWidth[5])); // wave 5
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[6] * _intTDMult, _intTDWave[7] * _intTDMult, TDDirection.LookForLows,  this._strWaveText[6],  this._boolPlotWaveLine[6], this._colorWaveLine[6], this._dsWaveLine[6], this._intWaveLineWidth[6])); // wave a
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[7] * _intTDMult, _intTDWave[8] * _intTDMult, TDDirection.LookForHighs, this._strWaveText[7],  this._boolPlotWaveLine[7], this._colorWaveLine[7], this._dsWaveLine[7], this._intWaveLineWidth[7])); // wave b
				this._alTDWaveRequirements.Add(new TDWaveRequirements(_intTDWave[8] * _intTDMult, 00,                         TDDirection.LookForLows,  this._strWaveText[8],  true, Color.Transparent, DashStyle.Solid, 5)); // wave c completed when closes below low close of wave a
			}
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            // Use this method for calculating your indicator values. Assign a value to each
            // plot below by replacing 'Close[0]' with your own formula.
			if (CurrentBar < this._intSlow * this._intElliottOscillatorFilterMultiplier) return;
			if (this._doublePointOffset == 0) this._doublePointOffset = SMA(Range(), 21)[0] * 0.15;
			this._intCurrentWave = this._alTDWaveHelper.Count - 1;

			if (this._intCurrentWave <= 0 && this._boolElliottOscillatorFilterON)
			{
				this.DetermineTrendDirection();
				if (this._boolIsWaveInTheDirectionOfTheTrend == false) // allow wave 1 only in the direction of the trend
				{
					if (this._intCurrentWave == 0) this.RemoveWave(TDWave.WaveStart);
					return;				
				}
			}
			
			if (this._intCurrentWave >= 0) // TD Wave in progress?
			{
				// validate wave in progress, rearrange if necessary
				switch (this._intCurrentWave) 
				{
					case (int)TDWave.WaveStart: // established start point, wave 1 in progress
						break;
						
						
					case (int)TDWave.Wave1: // wave 1 minimun requirement complete, wave 2 in progress
						this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart];
						if ((this.WaveType == TDWaveType.UpWave && Close[0] < this._CurrentWaveHelper.TDWaveClose) || 
							(this.WaveType == TDWaveType.DownWave && Close[0] > this._CurrentWaveHelper.TDWaveClose)) // close wave 2 low < start pivot/wave 1 low?
						{
							this.RemoveWave(TDWave.Wave1); // yes, remove wave 1 high pivot. start pivot will shift to this bar
						}
						break;
						
						
					case (int)TDWave.Wave2: // wave 2 minimun requirement complete, wave 3 in progress
						this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart];
						if ((this.WaveType == TDWaveType.UpWave && Close[0] < this._CurrentWaveHelper.TDWaveClose) || 
							(this.WaveType == TDWaveType.DownWave && Close[0] > this._CurrentWaveHelper.TDWaveClose)) // close wave 2 low < start pivot/wave 1 low?
						{
							this.RemoveWave(TDWave.Wave2); // yes, wave 2 low is now below start low; remove wave 2 pivot.
							this.RemoveWave(TDWave.Wave1); // remove wave 1 pivot. start pivot will shift to this bar
						}
						break;
						
						
					case (int)TDWave.Wave3: // wave 3 minimun requirement complete, wave 4 in progress
						this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2];
						if ((this.WaveType == TDWaveType.UpWave && Close[0] < this._CurrentWaveHelper.TDWaveClose) || 
							(this.WaveType == TDWaveType.DownWave && Close[0] > this._CurrentWaveHelper.TDWaveClose)) // close wave 4 low < wave 2 low?
						{
							this.RemoveWave(TDWave.Wave3); // yes, wave 4 low is now below wave 2 low; remove wave 3 pivot.
							this.RemoveWave(TDWave.Wave2); // remove wave 2 pivot. wave 2 pivot will shift to this bar
						}
						break;
						
						
					case (int)TDWave.Wave4: // wave 4 minimun requirement complete, wave 5 in progress
						this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2];
						if ((this.WaveType == TDWaveType.UpWave && Close[0] < this._CurrentWaveHelper.TDWaveClose) || 
							(this.WaveType == TDWaveType.DownWave && Close[0] > this._CurrentWaveHelper.TDWaveClose)) // close wave 4 low < wave 2 low?
						{
							this.RemoveWave(TDWave.Wave4); // yes, wave 4 low is now below wave 2 low; remove wave 4 pivot.
							this.RemoveWave(TDWave.Wave3); // remove wave 3 pivot. wave 2 pivot will shift to this bar
						}
						break;
						
						
					case (int)TDWave.Wave5: // wave 5 minimun requirement complete, wave A in progress
						this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave4];
						if ((this.WaveType == TDWaveType.UpWave && Close[0] < this._CurrentWaveHelper.TDWaveClose) || 
							(this.WaveType == TDWaveType.DownWave && Close[0] > this._CurrentWaveHelper.TDWaveClose)) // close wave A low < wave 4 low?
						{
							this.RemoveWave(TDWave.Wave5); // yes, wave A low is now below wave 4 low; remove wave 5 pivot.
							this.RemoveWave(TDWave.Wave4); // remove wave 4 pivot. wave 4 pivot will shift to this bar
						}
						break;
						
						
					case (int)TDWave.WaveA: // wave A minimun requirement complete, wave B in progress
						this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave4];
						if ((this.WaveType == TDWaveType.UpWave && Close[0] < this._CurrentWaveHelper.TDWaveClose) || 
							(this.WaveType == TDWaveType.DownWave && Close[0] > this._CurrentWaveHelper.TDWaveClose)) // close wave A low < wave 4 low?
						{
							// save wave A
							TDWaveHelper waveA = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveA];
							this.RemoveWave(TDWave.WaveA); // yes, wave A low is now below wave 4 low; remove wave A pivot.
							// save wave 5
							TDWaveHelper wave5 = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave5];
							this.RemoveWave(TDWave.Wave5); // remove wave 5 pivot. 
							
							// move wave 3 to wave 5
							TDWaveHelper wave3 = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3];
							wave3.TDWaveBar = wave5.TDWaveBar;
							wave3.TDWaveHigh = wave5.TDWaveHigh;
							wave3.TDWaveLow = wave5.TDWaveLow;
							wave3.TDWaveClose = wave5.TDWaveClose;
							this._alTDWaveHelper[(int)TDWave.Wave3] = wave3;
							this.DrawWaveName(wave3);

							// move wave 4 to wave A
							TDWaveHelper wave4 = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave4];
							wave4.TDWaveBar = waveA.TDWaveBar;
							wave4.TDWaveHigh = waveA.TDWaveHigh;
							wave4.TDWaveLow = waveA.TDWaveLow;
							wave4.TDWaveClose = waveA.TDWaveClose;
							this._alTDWaveHelper[(int)TDWave.Wave4] = wave4;
							this.DrawWaveName(wave4);
						}
						else
						{
							this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave5];
							if ((this.WaveType == TDWaveType.UpWave && Close[0] > this._CurrentWaveHelper.TDWaveClose) || 
								(this.WaveType == TDWaveType.DownWave && Close[0] < this._CurrentWaveHelper.TDWaveClose)) // close wave B high > wave 5 high?
							{
								this.RemoveWave(TDWave.WaveA); // yes, wave B high is now above wave 5 high; remove wave A pivot.
								this.RemoveWave(TDWave.Wave5); // remove wave 5 pivot. wave 5 pivot will shift to this bar
							}
						}
						break;
						
						
					case (int)TDWave.WaveB: // wave B minimun requirement complete, wave C in progress
						this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave5];
						if ((this.WaveType == TDWaveType.UpWave && Close[0] > this._CurrentWaveHelper.TDWaveClose) || 
							(this.WaveType == TDWaveType.DownWave && Close[0] < this._CurrentWaveHelper.TDWaveClose)) // close wave B high > wave 5 high?
						{
							this.RemoveWave(TDWave.WaveB); // yes, wave B high is now above wave 5 high; remove wave B pivot.
							this.RemoveWave(TDWave.WaveA); // remove wave A pivot. wave 5 pivot will shift to this bar
						}
						break;
					
					case (int)TDWave.WaveC: // wave C minimun requirement complete
						break;

					default:
						break;
				}
				this._intCurrentWave = this._alTDWaveHelper.Count - 1;
				this._CurrentWaveHelper = (TDWaveHelper)this._alTDWaveHelper[this._intCurrentWave];
			}
						
			this._NextWaveRequirements = (TDWaveRequirements)this._alTDWaveRequirements[this._intCurrentWave + 1];
			
			if (this._NextWaveRequirements.TDSingleWaveDirection == TDDirection.LookForHighs)
			{
				if (this._intCurrentWave >= 0 && Close[0] < this._CurrentWaveHelper.TDWaveClose) this.UpdateCurrentWaveHelper(); // found lower low
				if (this._intCurrentWave < 0 || this.CurrentBar >= (this._CurrentWaveHelper.TDWaveBar + this._NextWaveRequirements.TDWaveBars1)) // wait minimun number of bars before next requirement
					if (Close[0] == MAX(Close, this._NextWaveRequirements.TDWaveBars1)[0] && this.IsWaveValid()) 
						this.AddNewWaveHelper(); // found possible pivot high
			}
			else
			{
				if (this._intCurrentWave >= 0 && Close[0] > this._CurrentWaveHelper.TDWaveClose) this.UpdateCurrentWaveHelper(); // found higher high 
				if (this._intCurrentWave < 0 || this.CurrentBar >= (this._CurrentWaveHelper.TDWaveBar + this._NextWaveRequirements.TDWaveBars1)) // wait minimun number of bars before next requirement
					if (Close[0] == MIN(Close, this._NextWaveRequirements.TDWaveBars1)[0] && this.IsWaveValid()) 
						this.AddNewWaveHelper(); // found possible pivot low
			}
			
			if (this._alTDWaveHelper.Count > this._alTDWaveRequirements.Count - 1) // end of the pattern (wave C low break below wave A low)
			{
				this._alTDWaveHelper.Clear();
				this._strWave2RetracementLinesTag = string.Empty;
				this._strWave3PriceProjectionTag = string.Empty;
				this._strWave4RetracementLinesTag = string.Empty;
				this._strWave5PriceProjectionTag = string.Empty;
			}
			
			if (this._boolPlotWave2RetracementLines) this.DrawWave2RetracementLines();
			if (this._boolPlotWave3PriceProjection) this.DrawWave3PriceProjection();
			if (this._boolPlotWave4RetracementLines) this.DrawWave4RetracementLines();
			if (this._boolPlotWave5PriceProjection) this.DrawWave5PriceProjection();
		}

		private void AddNewWaveHelper()
		{
			TDWaveHelper wavehelper = new TDWaveHelper(this._alTDWaveHelper.Count, this.CurrentBar.ToString(), this.CurrentBar, High[0], Low[0], Close[0], this._NextWaveRequirements.TDWaveName, Color.White); 
			this._alTDWaveHelper.Add(wavehelper);			
			this.DrawWaveName(wavehelper);
//			this.BarColor = Color.Red;
		}

		private void DetermineTrendDirection()
		{
			this._doubleElliot = SMA(this._intFast*this._intElliottOscillatorFilterMultiplier)[0] -
				SMA(this._intSlow*this._intElliottOscillatorFilterMultiplier)[0];
			this._doubleElliotLookBackBars = SMA(this._intFast*this._intElliottOscillatorFilterMultiplier)[this._intLookBackBars] -
				SMA(this._intSlow*this._intElliottOscillatorFilterMultiplier)[this._intLookBackBars];
			
			if (this._WaveType == TDWaveType.UpWave)
				this._boolIsWaveInTheDirectionOfTheTrend = this._doubleElliot > 0 && this._doubleElliotLookBackBars > 0;
			else
				this._boolIsWaveInTheDirectionOfTheTrend = this._doubleElliot < 0 && this._doubleElliotLookBackBars < 0;
		}
		
		private void DrawWaveLines()
		{
			if (this._alTDWaveHelper.Count > 1)
				for (int index = 0; index < this._alTDWaveHelper.Count - 1; index++) 
				{
					if (((TDWaveRequirements)this._alTDWaveRequirements[index]).TDPlotWave)
					{
						TDWaveHelper wavehelper1 = ((TDWaveHelper)this._alTDWaveHelper[index]);
						TDWaveHelper wavehelper2 = ((TDWaveHelper)this._alTDWaveHelper[index + 1]);
						if (wavehelper2.TDWaveBar > wavehelper1.TDWaveBar)
							DrawLine(wavehelper1.TDWaveTag + "Line", 
								false, 
								(this.CurrentBar - wavehelper2.TDWaveBar) + (wavehelper2.TDWaveBar - wavehelper1.TDWaveBar), 
								(this.WaveType == TDWaveType.UpWave ? wavehelper1.TDWaveLow : wavehelper1.TDWaveHigh), 
								this.CurrentBar - wavehelper2.TDWaveBar, 
								(this.WaveType == TDWaveType.UpWave ? wavehelper2.TDWaveLow : wavehelper2.TDWaveHigh), 
								((TDWaveRequirements)this._alTDWaveRequirements[index]).TDWaveColor, //this._colorWaveLine, 
								((TDWaveRequirements)this._alTDWaveRequirements[index]).TDWaveDashStyle, //this._dsWaveLine, 
								((TDWaveRequirements)this._alTDWaveRequirements[index]).TDWaveWidth); //this._intWaveLineWidth
					}			
				}
		}
		
		#region Retracement Lines
		private void DrawWave2RetracementLines()
		{
			if (((this.IsWave2InProgress || this.IsWave3InProgress) 
				&& ((this.WaveType == TDWaveType.UpWave && this.IsCloseBelowPeakWave1) || (this.WaveType == TDWaveType.DownWave && this.IsCloseAboveTroughWave1))) == false) return;
			if (this._strWave2RetracementLinesTag.Length == 0) this._strWave2RetracementLinesTag = this.CurrentBar.ToString();
			double doubleWaveStart = 0;
			double doubleWave1 = 0;
			if (this.WaveType == TDWaveType.UpWave)
			{
				doubleWaveStart = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart]).TDWaveLow;
				doubleWave1 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveHigh;
			}
			else
			{
				doubleWaveStart = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart]).TDWaveHigh;
				doubleWave1 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveLow;
			}
			
			int intStartBar = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart]).TDWaveBar;
			int intEndBarHL = 0;
			if (this.IsWave2Completed) intEndBarHL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2]).TDWaveBar;
			if (this.IsWave3InProgress) intEndBarHL = 0;
			if (this.IsWave3InTheChart) intEndBarHL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveBar;
			int intEndBarVL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveBar;
			
			// Start wave horizontal line 
			this.DrawLine(this._strWave2RetracementLinesTag + "HL",
				false,
				intStartBar,
				doubleWaveStart,
				intEndBarHL,
				doubleWaveStart,
				this._colorRetracementLine[0],
				this._dsRetracementLine[0],
				this._intRetracementLineWidth[0]);

			// wave 1 vertical line 
			this.DrawLine(this._strWave2RetracementLinesTag + "VL",
				false,
				intEndBarVL,
				doubleWaveStart,
				intEndBarVL,
				doubleWave1,
				this._colorRetracementLine[0],
				this._dsRetracementLine[0],
				this._intRetracementLineWidth[0]);
			
			double doubleRange = (doubleWave1 - doubleWaveStart) * -1;
			double doubleY = 0;
			for (int index = 0; index < this._doubleRetracementLevels.Length; index++) 
			{
				// draw retracement line
				doubleY = doubleWave1 + doubleRange * this._doubleRetracementLevels[index];
				this.DrawLine(this._strWave2RetracementLinesTag + "RL" + index.ToString(),
					false,
					intEndBarVL,
					doubleY,
					intEndBarHL,
					doubleY,
					this._colorRetracementLine[index],
					this._dsRetracementLine[index],
					this._intRetracementLineWidth[index]);
				// draw retracement retracement text
				DrawText(this._strWave2RetracementLinesTag + "RLBL" + index.ToString(), 
					false, 
					String.Format("{0:F2}", this._doubleRetracementLevels[index] * 100) + "%", 
					intEndBarVL + 4, 
					doubleY + this._doublePointOffset / 5, 
					this._colorRetracementText[index], 
					new Font(FontFamily.GenericSansSerif, this._intRetracementFontSize[index], this._fsRetracementText, GraphicsUnit.Pixel), 
					StringAlignment.Center,
					Color.Transparent,
					Color.Transparent,
					0);
			}
		}
		
		private void DrawWave4RetracementLines()
		{
			if (((this.IsWave4InProgress || this.IsWave5InProgress) 
				&& ((this.WaveType == TDWaveType.UpWave && this.IsCloseBelowPeakWave3) || (this.WaveType == TDWaveType.DownWave && this.IsCloseAboveTroughWave3))) == false) return;
			if (this._strWave4RetracementLinesTag.Length == 0) this._strWave4RetracementLinesTag = this.CurrentBar.ToString();
			double doubleWave2 = 0;
			double doubleWave3 = 0;
			if (this.WaveType == TDWaveType.UpWave)
			{
				doubleWave2 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2]).TDWaveLow;
				doubleWave3 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveHigh;
			}
			else
			{
				doubleWave2 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2]).TDWaveHigh;
				doubleWave3 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveLow;
			}
			
			int intStartBar = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2]).TDWaveBar;
			int intEndBarHL = 0;
			if (this.IsWave4Completed) intEndBarHL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave4]).TDWaveBar;
			if (this.IsWave5InProgress) intEndBarHL = 0;
			if (this.IsWave5InTheChart) intEndBarHL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave5]).TDWaveBar;
			int intEndBarVL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveBar;
			
			// Start wave horizontal line 
			this.DrawLine(this._strWave4RetracementLinesTag + "HL",
				false,
				intStartBar,
				doubleWave2,
				intEndBarHL,
				doubleWave2,
				this._colorRetracementLine[0],
				this._dsRetracementLine[0],
				this._intRetracementLineWidth[0]);

			// wave 1 vertical line 
			this.DrawLine(this._strWave4RetracementLinesTag + "VL",
				false,
				intEndBarVL,
				doubleWave2,
				intEndBarVL,
				doubleWave3,
				this._colorRetracementLine[0],
				this._dsRetracementLine[0],
				this._intRetracementLineWidth[0]);
			
			double doubleRange = (doubleWave3 - doubleWave2) * -1;
			double doubleY = 0;
			for (int index = 0; index < this._doubleRetracementLevels.Length; index++) 
			{
				// draw retracement line
				doubleY = doubleWave3 + doubleRange * this._doubleRetracementLevels[index];
				this.DrawLine(this._strWave4RetracementLinesTag + "RL" + index.ToString(),
					false,
					intEndBarVL,
					doubleY,
					intEndBarHL,
					doubleY,
					this._colorRetracementLine[index],
					this._dsRetracementLine[index],
					this._intRetracementLineWidth[index]);
				// draw retracement retracement text
				DrawText(this._strWave4RetracementLinesTag + "RLBL" + index.ToString(), 
					false, 
					String.Format("{0:F2}", this._doubleRetracementLevels[index] * 100) + "%", 
					intEndBarVL + 4, 
					doubleY + this._doublePointOffset / 5, 
					this._colorRetracementText[index], 
					new Font(FontFamily.GenericSansSerif, this._intRetracementFontSize[index], this._fsRetracementText, GraphicsUnit.Pixel), 
					StringAlignment.Center,
					Color.Transparent,
					Color.Transparent,
					0);
			}
		}
		#endregion
		
		#region Prive Projection Lines
		private void DrawWave3PriceProjection()
		{
			if (this.IsWave3InProgress == false) return;
			if (this._strWave3PriceProjectionTag.Length == 0) this._strWave3PriceProjectionTag = this.CurrentBar.ToString();
			double doubleWaveStart = 0;
			double doubleWave1 = 0;
			if (this.WaveType == TDWaveType.UpWave)
			{
				doubleWaveStart = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart]).TDWaveLow;
				doubleWave1 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveHigh;
			}
			else
			{
				doubleWaveStart = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart]).TDWaveHigh;
				doubleWave1 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveLow;
			}
			
			double doubleWave3PriceProjection = 0;
			if (this.WaveType == TDWaveType.UpWave)
				doubleWave3PriceProjection = doubleWaveStart + (doubleWave1 - doubleWaveStart) * this._doubleWave3Factor1;
			else
				doubleWave3PriceProjection = doubleWaveStart - (doubleWaveStart - doubleWave1) * this._doubleWave3Factor1;
			
			int intStartBar = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveStart]).TDWaveBar;
			int intEndBarHL = 0;
			if (this.IsWave3InTheChart) intEndBarHL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveBar;
			
			// Start wave horizontal line 
			this.DrawLine(this._strWave2RetracementLinesTag + "HL",
				false,
				intStartBar,
				doubleWaveStart,
				intEndBarHL,
				doubleWaveStart,
				this._colorRetracementLine[0],
				this._dsRetracementLine[0],
				this._intRetracementLineWidth[0]);

			// wave 3 vertical line 
			this.DrawLine(this._strWave3PriceProjectionTag + "VL",
				false,
				intEndBarHL,
				doubleWaveStart,
				intEndBarHL,
				doubleWave3PriceProjection,
				this._colorWave3PriceProjectionLines,
				this._dsWave3PriceProjectionLines,
				this._intWave3PriceProjectionLinesWidth);

			// wave 3 horizontal line 
			this.DrawLine(this._strWave3PriceProjectionTag + "HL",
				false,
				intEndBarHL + this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveBar,
				doubleWave3PriceProjection,
				0,
				doubleWave3PriceProjection,
				this._colorWave3PriceProjectionLines,
				this._dsWave3PriceProjectionLines,
				this._intWave3PriceProjectionLinesWidth);
		}
		
		private void DrawWave5PriceProjection()
		{
			if (this.IsWave5InProgress == false) return;
			if (this._strWave5PriceProjectionTag.Length == 0) this._strWave5PriceProjectionTag = this.CurrentBar.ToString();
			double doubleWave2 = 0;
			double doubleWave3 = 0;
			if (this.WaveType == TDWaveType.UpWave)
			{
				doubleWave2 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2]).TDWaveLow;
				doubleWave3 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveHigh;
			}
			else
			{
				doubleWave2 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2]).TDWaveHigh;
				doubleWave3 = ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveLow;
			}
			
			double doubleWave5PriceProjection = 0;
			if (this.WaveType == TDWaveType.UpWave)
				doubleWave5PriceProjection = doubleWave2 + (doubleWave3 - doubleWave2) * 1.618;
			else
				doubleWave5PriceProjection = doubleWave2 - (doubleWave2 - doubleWave3) * 1.618;
			
			int intStartBar = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave2]).TDWaveBar;
			int intEndBarHL = 0;
			if (this.IsWave5InTheChart) intEndBarHL = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave5]).TDWaveBar;
			
			// Wave 2 horizontal line 
			this.DrawLine(this._strWave4RetracementLinesTag + "HL",
				false,
				intStartBar,
				doubleWave2,
				intEndBarHL,
				doubleWave2,
				this._colorRetracementLine[0],
				this._dsRetracementLine[0],
				this._intRetracementLineWidth[0]);

			// wave 5 vertical line 
			this.DrawLine(this._strWave5PriceProjectionTag + "VL",
				false,
				intEndBarHL,
				doubleWave2,
				intEndBarHL,
				doubleWave5PriceProjection,
				this._colorWave3PriceProjectionLines,
				this._dsWave3PriceProjectionLines,
				this._intWave3PriceProjectionLinesWidth);

			// wave 5 horizontal line 
			this.DrawLine(this._strWave5PriceProjectionTag + "HL",
				false,
				intEndBarHL + this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveBar,
				doubleWave5PriceProjection,
				0,
				doubleWave5PriceProjection,
				this._colorWave3PriceProjectionLines,
				this._dsWave3PriceProjectionLines,
				this._intWave3PriceProjectionLinesWidth);
		}
		#endregion
		
		private void DrawWaveName(TDWaveHelper wavehelper)
		{
			DrawText(wavehelper.TDWaveTag, 
				false, // autoscale
				wavehelper.TDWaveName, // text
				this.CurrentBar - wavehelper.TDWaveBar, // bars ago
				(this.WaveType == TDWaveType.UpWave ? wavehelper.TDWaveLow - this._doublePointOffset : wavehelper.TDWaveHigh + this._doublePointOffset), 
				this._colorWaveText[wavehelper.TDIndex], 
				new Font(FontFamily.GenericSansSerif, this._intWaveTextFontSize[wavehelper.TDIndex], this._fsWaveText[wavehelper.TDIndex], GraphicsUnit.Pixel), 
				StringAlignment.Center,
				Color.Transparent,
				Color.Transparent,
				0);
			this.DrawWaveLines();
		}

		private bool IsWaveValid()
		{
			bool boolReturn = true;
			if (this._intCurrentWave >= 0)
			{
				switch (this._intCurrentWave) 
				{
					case (int)TDWave.WaveStart: // established start point, wave 1 in progress
						if (this.WaveType == TDWaveType.UpWave)
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
						else
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
						break;
					case (int)TDWave.Wave1: // wave 1 minimun requirement complete, wave 2 in progress
						if (this.WaveType == TDWaveType.UpWave)
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
						else
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
						break;
					case (int)TDWave.Wave2: // wave 2 minimun requirement complete, wave 3 in progress
						if (this.WaveType == TDWaveType.UpWave)
						{
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] > ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveClose; // wave 3 high > wave 1 high
						}
						else
						{
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] < ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveClose; // wave 3 high > wave 1 high
						}
						break;
					case (int)TDWave.Wave3: // wave 3 minimun requirement complete, wave 4 in progress
						if (this.WaveType == TDWaveType.UpWave)
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
						else
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
						break;
					case (int)TDWave.Wave4: // wave 4 minimun requirement complete, wave 5 in progress
						if (this.WaveType == TDWaveType.UpWave)
						{
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] > ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveClose; // wave 5 high > wave 3 high
						}
						else
						{
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] < ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveClose; // wave 5 high > wave 3 high
						}
						break;
					case (int)TDWave.Wave5: // wave 5 minimun requirement complete, wave A in progress
						if (this.WaveType == TDWaveType.UpWave)
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
						else
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
						break;
					case (int)TDWave.WaveA: // wave A minimun requirement complete, wave B in progress
						if (this.WaveType == TDWaveType.UpWave)
						{
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] < ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave5]).TDWaveClose; // wave B high < wave 5 high
						}
						else
						{
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] > ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave5]).TDWaveClose; // wave B high < wave 5 high
						}
						break;
					case (int)TDWave.WaveB: // wave B minimun requirement complete, wave C in progress
						if (this.WaveType == TDWaveType.UpWave)
						{
							boolReturn = Close[0] < this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] < ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveA]).TDWaveClose; // wave C Low < wave A low
						}
						else
						{
							boolReturn = Close[0] > this._CurrentWaveHelper.TDWaveClose;	
							boolReturn = boolReturn && Close[0] > ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.WaveA]).TDWaveClose; // wave C Low < wave A low
						}
						break;
					default:
						boolReturn = true;
						break;
				}
			}
			return boolReturn;
		}

		#region Remove Objects
		private void RemoveWave2RetracementLines()
		{
			if (this._strWave2RetracementLinesTag.Length > 0)
			{
				this.RemoveDrawObject(this._strWave2RetracementLinesTag + "HL");
				this.RemoveDrawObject(this._strWave2RetracementLinesTag + "VL");
				for (int index = 0; index < this._doubleRetracementLevels.Length; index++) 
				{
					this.RemoveDrawObject(this._strWave2RetracementLinesTag + "RL" + index.ToString());
					this.RemoveDrawObject(this._strWave2RetracementLinesTag + "RLBL" + index.ToString());
				}
				this._strWave2RetracementLinesTag = string.Empty;
			}
		}

		private void RemoveWave3PriceProjectionLines()
		{
			if (this._strWave3PriceProjectionTag.Length > 0)
			{
				this.RemoveDrawObject(this._strWave3PriceProjectionTag + "HL");
				this.RemoveDrawObject(this._strWave3PriceProjectionTag + "VL");
				this._strWave3PriceProjectionTag = string.Empty;
			}
		}

		private void RemoveWave4RetracementLines()
		{
			if (this._strWave4RetracementLinesTag.Length > 0)
			{
				this.RemoveDrawObject(this._strWave4RetracementLinesTag + "HL");
				this.RemoveDrawObject(this._strWave4RetracementLinesTag + "VL");
				for (int index = 0; index < this._doubleRetracementLevels.Length; index++) 
				{
					this.RemoveDrawObject(this._strWave4RetracementLinesTag + "RL" + index.ToString());
					this.RemoveDrawObject(this._strWave4RetracementLinesTag + "RLBL" + index.ToString());
				}
				this._strWave4RetracementLinesTag = string.Empty;
			}
		}

		private void RemoveWave5PriceProjectionLines()
		{
			if (this._strWave5PriceProjectionTag.Length > 0)
			{
				this.RemoveDrawObject(this._strWave5PriceProjectionTag + "HL");
				this.RemoveDrawObject(this._strWave5PriceProjectionTag + "VL");
				this._strWave5PriceProjectionTag = string.Empty;
			}
		}
		
		private void RemoveWave(int intWave)
		{
			this.RemoveDrawObject(((TDWaveHelper)this._alTDWaveHelper[intWave]).TDWaveTag);
			if (intWave > 0)
			{
				this.RemoveDrawObject(((TDWaveHelper)this._alTDWaveHelper[intWave - 1]).TDWaveTag + "Line");
				if (intWave == (int)TDWave.Wave1) this.RemoveWave2RetracementLines();
				if (intWave == (int)TDWave.Wave2) this.RemoveWave3PriceProjectionLines();
				if (intWave == (int)TDWave.Wave3) this.RemoveWave4RetracementLines();
				if (intWave == (int)TDWave.Wave4) this.RemoveWave5PriceProjectionLines();
			}
			this._alTDWaveHelper.RemoveAt(intWave);
		}
		
		private void RemoveWave(TDWave wave)
		{
			this.RemoveDrawObject(((TDWaveHelper)this._alTDWaveHelper[(int)wave]).TDWaveTag);
			if ((int)wave > 0)
			{
				this.RemoveDrawObject(((TDWaveHelper)this._alTDWaveHelper[((int)wave) - 1]).TDWaveTag + "Line");
				if (wave == TDWave.Wave1) this.RemoveWave2RetracementLines();
				if (wave == TDWave.Wave2) this.RemoveWave3PriceProjectionLines();
				if (wave == TDWave.Wave3) this.RemoveWave4RetracementLines();
				if (wave == TDWave.Wave4) this.RemoveWave5PriceProjectionLines();
			}
			this._alTDWaveHelper.RemoveAt((int)wave);
		}
		#endregion
		
		private void UpdateCurrentWaveHelper()
		{
			if (this._intCurrentWave == 2 || this._intCurrentWave == 4)
			{
				double doubleWaveClose = ((TDWaveHelper)this._alTDWaveHelper[this._intCurrentWave - 1]).TDWaveClose; // get close of 2 pivots ago
				int intPeriod = this.CurrentBar - ((TDWaveHelper)this._alTDWaveHelper[this._intCurrentWave - 1]).TDWaveBar;
				if (this._NextWaveRequirements.TDSingleWaveDirection == TDDirection.LookForHighs) // check for swing high that will require a wave to shift to the right
				{
					// plotted a pivot high, plotted a pivot low, while trying to find the next pivot high price went above 
					// prior pivot high, didn't make the minimun requirements, came back and broke below the pivot low
					if (doubleWaveClose < MAX(Close, intPeriod)[0]) 
					{
						double doubleCloseHigh = MAX(Close, intPeriod)[0];
						for (int index = 1; index < intPeriod; index++) 
						{
							if (Close[index] == doubleCloseHigh)
							{
								TDWaveHelper wavehelper = ((TDWaveHelper)this._alTDWaveHelper[this._intCurrentWave - 1]);
								wavehelper.TDWaveBar = this.CurrentBar - index;
								wavehelper.TDWaveHigh = High[index];
								wavehelper.TDWaveLow = Low[index];
								wavehelper.TDWaveClose = Close[index];
								this._alTDWaveHelper[this._intCurrentWave - 1] = wavehelper;
								this.DrawWaveName(wavehelper);
								break;
							}
						}
					}
				}
				else
				{
					// plotted a pivot low, plotted a pivot high, while trying to find the next pivot low price went below 
					// prior pivot low, didn't make the minimun requirements, came back and broke above the pivot high
					if (doubleWaveClose > MIN(Close, intPeriod)[0]) 
					{
						double doubleCloseLow = MIN(Close, intPeriod)[0];
						for (int index = 1; index < intPeriod; index++) 
						{
							if (Close[index] == doubleCloseLow)
							{
								TDWaveHelper wavehelper = ((TDWaveHelper)this._alTDWaveHelper[this._intCurrentWave - 1]);
								wavehelper.TDWaveBar = this.CurrentBar - index;
								wavehelper.TDWaveHigh = High[index];
								wavehelper.TDWaveLow = Low[index];
								wavehelper.TDWaveClose = Close[index];
								this._alTDWaveHelper[this._intCurrentWave - 1] = wavehelper;
								this.DrawWaveName(wavehelper);
								break;
							}
						}
					}
				}
			}
			
			this._CurrentWaveHelper.TDWaveBar = this.CurrentBar;
			this._CurrentWaveHelper.TDWaveHigh = High[0];
			this._CurrentWaveHelper.TDWaveLow = Low[0];
			this._CurrentWaveHelper.TDWaveClose = Close[0];
			this._alTDWaveHelper[this._intCurrentWave] = this._CurrentWaveHelper; // move pivot up or down
			this.DrawWaveName(this._CurrentWaveHelper);
		}
		
        #region Properties

		#region Private Properties
        private bool IsCloseAboveTroughWave1
        {
            get { return Close[0] >= ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveLow; }
        }
        private bool IsCloseAboveTroughWave3
        {
            get { return Close[0] >= ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveLow; }
        }
        private bool IsCloseBelowPeakWave1
        {
            get { return Close[0] <= ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave1]).TDWaveHigh; }
        }
        private bool IsCloseBelowPeakWave3
        {
            get { return Close[0] <= ((TDWaveHelper)this._alTDWaveHelper[(int)TDWave.Wave3]).TDWaveHigh; }
        }
        private bool IsWave2Completed
        {
            get { return this._alTDWaveHelper.Count > 3; }
        }
        private bool IsWave2InProgress
        {
            get { return this._alTDWaveHelper.Count == 2 || this._alTDWaveHelper.Count == 3; }
        }
        private bool IsWave3Completed
        {
            get { return this._alTDWaveHelper.Count > 4; }
        }
        private bool IsWave3InProgress
        {
            get { return this._alTDWaveHelper.Count == 3 || this._alTDWaveHelper.Count == 4; }
        }
        private bool IsWave3InTheChart
        {
            get { return this._alTDWaveHelper.Count > 3; }
        }
        private bool IsWave4Completed
        {
            get { return this._alTDWaveHelper.Count > 5; }
        }
        private bool IsWave4InProgress
        {
            get { return this._alTDWaveHelper.Count == 4 || this._alTDWaveHelper.Count == 5; }
        }
        private bool IsWave5InProgress
        {
            get { return this._alTDWaveHelper.Count == 5 || this._alTDWaveHelper.Count == 6; }
        }
        private bool IsWave5InTheChart
        {
            get { return this._alTDWaveHelper.Count > 5; }
        }
		#endregion
		
		#region General Parameters
        [Description("Increase wave level by this number.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Multiplier")]
        public int Multiplier
        {
            get { return _intTDMult ; }
            set { _intTDMult  = Math.Max(1, value); }
        }
		
		[Description("Wave level 0.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level 0")]
        public int WaveStart
        {
            get { return _intTDWave[0] ; }
            set { _intTDWave[0]  = Math.Max(4, value); }
        }
		
        [Description("Wave level 1.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level 1")]
        public int WaveLevel1
        {
            get { return _intTDWave[1]; }
            set { _intTDWave[1] = Math.Max(4, value); }
        }
		
        [Description("Wave level 2.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level 2")]
        public int WaveLevel2
        {
            get { return _intTDWave[2]; }
            set { _intTDWave[2] = Math.Max(4, value); }
        }
		
        [Description("Wave level 3.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level 3")]
        public int WaveLevel3
        {
            get { return _intTDWave[3]; }
            set { _intTDWave[3] = Math.Max(4, value); }
        }
		
        [Description("Wave level 4.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level 4")]
        public int WaveLevel4
        {
            get { return _intTDWave[4]; }
            set { _intTDWave[4] = Math.Max(4, value); }
        }
		
        [Description("Wave level 5.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level 5")]
        public int WaveLevel5
        {
            get { return _intTDWave[5]; }
            set { _intTDWave[5] = Math.Max(4, value); }
        }
		
        [Description("Wave level A.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level A")]
        public int WaveLevelA
        {
            get { return _intTDWave[6]; }
            set { _intTDWave[6] = Math.Max(4, value); }
        }
		
        [Description("Wave level B.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level B")]
        public int WaveLevelB
        {
            get { return _intTDWave[7]; }
            set { _intTDWave[7] = Math.Max(4, value); }
        }
		
        [Description("Wave level C.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave level C")]
        public int WaveLevelC
        {
            get { return _intTDWave[8]; }
            set { _intTDWave[8] = Math.Max(4, value); }
        }
		
        [Description("Wave 0 text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave 0 text")]
        public string Wave0Text
        {
            get { return _strWaveText[0]; }
            set { _strWaveText[0] = value; }
        }
		
        [Description("Wave 1 text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave 1 text")]
        public string Wave1Text
        {
            get { return _strWaveText[1]; }
            set { _strWaveText[1] = value; }
        }
		
        [Description("Wave 2 text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave 2 text")]
        public string Wave2Text
        {
            get { return _strWaveText[2]; }
            set { _strWaveText[2] = value; }
        }
		
        [Description("Wave 3 text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave 3 text")]
        public string Wave3Text
        {
            get { return _strWaveText[3]; }
            set { _strWaveText[3] = value; }
        }
		
        [Description("Wave 4 text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave 4 text")]
        public string Wave4Text
        {
            get { return _strWaveText[4]; }
            set { _strWaveText[4] = value; }
        }
		
        [Description("Wave 5 text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave 5 text")]
        public string Wave5Text
        {
            get { return _strWaveText[5]; }
            set { _strWaveText[5] = value; }
        }
		
        [Description("Wave A text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave A text")]
        public string WaveAText
        {
            get { return _strWaveText[6]; }
            set { _strWaveText[6] = value; }
        }
		
        [Description("Wave B text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave B text")]
        public string WaveBText
        {
            get { return _strWaveText[7]; }
            set { _strWaveText[7] = value; }
        }
		
        [Description("Wave C text.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave C text")]
        public string WaveCText
        {
            get { return _strWaveText[8]; }
            set { _strWaveText[8] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Text Color.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave Text Color")]
        public Color WaveTextColor
        {
            get { return _colorWaveText[0]; }
            set 
			{ 
				for (int index = 0; index < _colorWaveText.Length; index++) 
					_colorWaveText[index] = value; 
			}
        }
		[Browsable(false)]
		public string WaveTextColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveText[0] ); }
			set { _colorWaveText[0] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        [Description("Wave Text Font Size.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave Text Font Size")]
        public int WaveTextFontSize
        {
            get { return _intWaveTextFontSize[0]; }
            set 
			{ 
				for (int index = 0; index < _colorWaveText.Length; index++) 
					_intWaveTextFontSize[index] = Math.Max(5, value); 
			}
        }
		
        [Description("Wave Text Font Style.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave Text Font Style")]
        public FontStyle WaveTextFontStyle
        {
            get { return _fsWaveText[0]; }
            set 
			{ 
				for (int index = 0; index < _colorWaveText.Length; index++) 
					_fsWaveText[index] = value; 
			}
        }
		
        [Description("Wave Type.")]
        [Category("00-General Parameters")]
		[Gui.Design.DisplayName("Wave Type")]
        public TDWaveType WaveType
        {
            get { return _WaveType; }
            set 
			{ 
				_WaveType = value; 
				if (this._intWaveType > 0)
				{
					if (this._WaveType == TDWaveType.DownWave)
					{
						for (int index = 0; index < this._colorWaveLine.Length; index++) 
						{
							this._colorWaveLine[index] = Color.Red;
							this._colorWaveText[index] = Color.Red;
						}
						for (int index = 0; index < this._doubleRetracementLevels.Length; index++) 
						{
							this._colorRetracementLine[index] = Color.Red;
							this._colorRetracementText[index] = Color.Red;
						}
					}
					else
					{
						for (int index = 0; index < this._colorWaveLine.Length; index++) 
						{
							this._colorWaveLine[index] = Color.Green;
							this._colorWaveText[index] = Color.Green;
						}
						for (int index = 0; index < this._doubleRetracementLevels.Length; index++) 
						{
							this._colorRetracementLine[index] = Color.Green;
							this._colorRetracementText[index] = Color.Green;
						}
					}
					this._colorRetracementLine[3] = Color.Yellow;
					this._colorRetracementText[3] = Color.Yellow;
					this._colorRetracementLine[5] = Color.Magenta;
					this._colorRetracementText[5] = Color.Magenta;
				}
				this._intWaveType++;
			}
        }
		#endregion		
	
		#region Wave Lines
		
		#region Wave 1 Line Parameters
        [Description("Plot Wave Line.")]
        [Category("01-Wave 1 Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWave1Line
        {
            get { return this._boolPlotWaveLine[0]; }
            set { this._boolPlotWaveLine[0] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("01-Wave 1 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color Wave1LineColor
        {
            get { return _colorWaveLine[0] ; }
            set { _colorWaveLine[0]   = value; }
        }
		[Browsable(false)]
		public string Wave1LineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[0]); }
			set { _colorWaveLine[0] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("01-Wave 1 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle Wave1LineDashStyle
        {
            get { return _dsWaveLine[0]; }
            set { _dsWaveLine[0] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("01-Wave 1 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int Wave1LineWidth
        {
            get { return _intWaveLineWidth[0]; }
            set { _intWaveLineWidth[0] = Math.Max(0, value); }
        }
        #endregion
		
		#region Wave 2 Line Parameters
        [Description("Plot Wave Line.")]
        [Category("02-Wave 2 Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWave2Line
        {
            get { return this._boolPlotWaveLine[1]; }
            set { this._boolPlotWaveLine[1] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("02-Wave 2 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color Wave2LineColor
        {
            get { return _colorWaveLine[1] ; }
            set { _colorWaveLine[1]   = value; }
        }
		[Browsable(false)]
		public string Wave2LineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[1]); }
			set { _colorWaveLine[1] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("02-Wave 2 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle Wave2LineDashStyle
        {
            get { return _dsWaveLine[1]; }
            set { _dsWaveLine[1] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("02-Wave 2 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int Wave2LineWidth
        {
            get { return _intWaveLineWidth[1]; }
            set { _intWaveLineWidth[1] = Math.Max(0, value); }
        }
        #endregion
		
		#region Wave 3 Line Parameters
        [Description("Plot Wave Line.")]
        [Category("03-Wave 3 Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWave3Line
        {
            get { return this._boolPlotWaveLine[2]; }
            set { this._boolPlotWaveLine[2] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("03-Wave 3 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color Wave3LineColor
        {
            get { return _colorWaveLine[2] ; }
            set { _colorWaveLine[2]   = value; }
        }
		[Browsable(false)]
		public string Wave3LineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[2]); }
			set { _colorWaveLine[2] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("03-Wave 3 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle Wave3LineDashStyle
        {
            get { return _dsWaveLine[2]; }
            set { _dsWaveLine[2] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("03-Wave 3 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int Wave3LineWidth
        {
            get { return _intWaveLineWidth[2]; }
            set { _intWaveLineWidth[2] = Math.Max(0, value); }
        }
        #endregion
		
		#region Wave 4 Line Parameters
        [Description("Plot Wave Line.")]
        [Category("04-Wave 4 Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWave4Line
        {
            get { return this._boolPlotWaveLine[3]; }
            set { this._boolPlotWaveLine[3] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("04-Wave 4 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color Wave4LineColor
        {
            get { return _colorWaveLine[3] ; }
            set { _colorWaveLine[3]   = value; }
        }
		[Browsable(false)]
		public string Wave4LineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[3]); }
			set { _colorWaveLine[3] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("04-Wave 4 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle Wave4LineDashStyle
        {
            get { return _dsWaveLine[3]; }
            set { _dsWaveLine[3] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("04-Wave 4 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int Wave4LineWidth
        {
            get { return _intWaveLineWidth[3]; }
            set { _intWaveLineWidth[3] = Math.Max(0, value); }
        }
        #endregion
		
		#region Wave 5 Line Parameters
        [Description("Plot Wave Line.")]
        [Category("05-Wave 5 Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWave5Line
        {
            get { return this._boolPlotWaveLine[4]; }
            set { this._boolPlotWaveLine[4] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("05-Wave 5 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color Wave5LineColor
        {
            get { return _colorWaveLine[4] ; }
            set { _colorWaveLine[4]   = value; }
        }
		[Browsable(false)]
		public string Wave5LineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[4]); }
			set { _colorWaveLine[4] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("05-Wave 5 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle Wave5LineDashStyle
        {
            get { return _dsWaveLine[4]; }
            set { _dsWaveLine[4] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("05-Wave 5 Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int Wave5LineWidth
        {
            get { return _intWaveLineWidth[4]; }
            set { _intWaveLineWidth[4] = Math.Max(0, value); }
        }
        #endregion
		
		#region Wave A Line Parameters
        [Description("Plot Wave Line.")]
        [Category("06-Wave A Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWaveALine
        {
            get { return this._boolPlotWaveLine[5]; }
            set { this._boolPlotWaveLine[5] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("06-Wave A Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color WaveALineColor
        {
            get { return _colorWaveLine[5] ; }
            set { _colorWaveLine[5] = value; }
        }
		[Browsable(false)]
		public string WaveALineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[5]); }
			set { _colorWaveLine[5] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("06-Wave A Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle WaveALineDashStyle
        {
            get { return _dsWaveLine[5]; }
            set { _dsWaveLine[5] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("06-Wave A Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int WaveALineWidth
        {
            get { return _intWaveLineWidth[5]; }
            set { _intWaveLineWidth[5] = Math.Max(0, value); }
        }
        #endregion
		
		#region Wave B Line Parameters
        [Description("Plot Wave Line.")]
        [Category("07-Wave B Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWaveBLine
        {
            get { return this._boolPlotWaveLine[6]; }
            set { this._boolPlotWaveLine[6] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("07-Wave B Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color WaveBLineColor
        {
            get { return _colorWaveLine[6] ; }
            set { _colorWaveLine[6] = value; }
        }
		[Browsable(false)]
		public string WaveBLineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[6]); }
			set { _colorWaveLine[6] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("07-Wave B Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle WaveBLineDashStyle
        {
            get { return _dsWaveLine[6]; }
            set { _dsWaveLine[6] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("07-Wave B Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int WaveBLineWidth
        {
            get { return _intWaveLineWidth[6]; }
            set { _intWaveLineWidth[6] = Math.Max(0, value); }
        }
        #endregion
		
		#region Wave C Line Parameters
        [Description("Plot Wave Line.")]
        [Category("08-Wave C Line Parameters")]
		[Gui.Design.DisplayName("Plot Wave Line")]
        public bool PlotWaveCLine
        {
            get { return this._boolPlotWaveLine[7]; }
            set { this._boolPlotWaveLine[7] = value; }
        }

		[XmlIgnore()]
        [Description("Wave Line Color.")]
        [Category("08-Wave C Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Color")]
        public Color WaveCLineColor
        {
            get { return _colorWaveLine[7] ; }
            set { _colorWaveLine[7] = value; }
        }
		[Browsable(false)]
		public string WaveCLineColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWaveLine[7]); }
			set { _colorWaveLine[7] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
				
        [Description("Wave Line Dash Style.")]
        [Category("08-Wave C Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Dash Style")]
        public DashStyle WaveCLineDashStyle
        {
            get { return _dsWaveLine[7]; }
            set { _dsWaveLine[7] = value; }
        }
		
        [Description("Wave Line Width.")]
        [Category("08-Wave C Line Parameters")]
		[Gui.Design.DisplayName("Wave Line Width")]
        public int WaveCLineWidth
        {
            get { return _intWaveLineWidth[7]; }
            set { _intWaveLineWidth[7] = Math.Max(0, value); }
        }
        #endregion
		
		#endregion
		
        #region Retracement Lines
	    [Description("Draw Wave 2 Retracement Lines.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Draw Wave 2 Retracement Lines")]
        public bool PlotWave2Retracements
        {
            get { return this._boolPlotWave2RetracementLines; }
            set { this._boolPlotWave2RetracementLines = value; }
        }
	    [Description("Draw Wave 4 Retracement Lines.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Draw Wave 4 Retracement Lines")]
        public bool PlotWave4Retracements
        {
            get { return this._boolPlotWave4RetracementLines; }
            set { this._boolPlotWave4RetracementLines = value; }
        }
	    [Description("Retracement Lines Font Style.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Retracement Lines Font Style")]
        public FontStyle RetracementTextFontStyle
        {
            get { return this._fsRetracementText; }
            set { this._fsRetracementText = value; }
        }
		
		#region Line Color
		[XmlIgnore()]
        [Description("Retracement Line 0% Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 0% Color")]
        public Color RetracementLineLevel00
        {
            get { return _colorRetracementLine[0]; }
            set { _colorRetracementLine[0]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel00Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementLine[0]); }
			set { _colorRetracementLine[0] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 23% Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 23% Color")]
        public Color RetracementLineLevel23
        {
            get { return _colorRetracementLine[1]; }
            set { _colorRetracementLine[1]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel23Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementLine[1]); }
			set { _colorRetracementLine[1] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 38% Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 38% Color")]
        public Color RetracementLineLevel38
        {
            get { return _colorRetracementLine[2]; }
            set { _colorRetracementLine[2]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel38Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementLine[2]); }
			set { _colorRetracementLine[2] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 50% Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 50% Color")]
        public Color RetracementLineLevel50
        {
            get { return _colorRetracementLine[3]; }
            set { _colorRetracementLine[3]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel50Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementLine[3]); }
			set { _colorRetracementLine[3] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 62.8% Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 62.8% Color")]
        public Color RetracementLineLevel62
        {
            get { return _colorRetracementLine[4]; }
            set { _colorRetracementLine[4]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel62Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementLine[4]); }
			set { _colorRetracementLine[4] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 76.4% Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 76.4% Color")]
        public Color RetracementLineLevel76
        {
            get { return _colorRetracementLine[5]; }
            set { _colorRetracementLine[5]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel76Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementLine[5]); }
			set { _colorRetracementLine[5] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 100% Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 100% Color")]
        public Color RetracementLineLevel100
        {
            get { return _colorRetracementLine[6]; }
            set { _colorRetracementLine[6]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel100Serialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementLine[6]); }
			set { _colorRetracementLine[6] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
		
		#region Text Color
		[XmlIgnore()]
        [Description("Retracement Line 0% Text Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 0% Text Color")]
        public Color RetracementLineLevel00Text
        {
            get { return _colorRetracementText[0]; }
            set { _colorRetracementText[0]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel00TextSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementText[0]); }
			set { _colorRetracementText[0] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 23% Text Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 23% Text Color")]
        public Color RetracementLineLevel23Text
        {
            get { return _colorRetracementText[1]; }
            set { _colorRetracementText[1]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel23TextSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementText[1]); }
			set { _colorRetracementText[1] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 38% Text Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 38% Text Color")]
        public Color RetracementLineLevel38Text
        {
            get { return _colorRetracementText[2]; }
            set { _colorRetracementText[2]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel38TextSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementText[2]); }
			set { _colorRetracementText[2] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 50% Text Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 50% Text Color")]
        public Color RetracementLineLevel50Text
        {
            get { return _colorRetracementText[3]; }
            set { _colorRetracementText[3]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel50TextSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementText[3]); }
			set { _colorRetracementText[3] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 62.8% Text Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 62.8% Text Color")]
        public Color RetracementLineLevel62Text
        {
            get { return _colorRetracementText[4]; }
            set { _colorRetracementText[4]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel62TextSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementText[4]); }
			set { _colorRetracementText[4] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 76.4% Text Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 76.4% Text Color")]
        public Color RetracementLineLevel76Text
        {
            get { return _colorRetracementText[5]; }
            set { _colorRetracementText[5]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel76TextSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementText[5]); }
			set { _colorRetracementText[5] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		[XmlIgnore()]
        [Description("Retracement Line 100% Text Color.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 100% Text Color")]
        public Color RetracementLineLevel100Text
        {
            get { return _colorRetracementText[6]; }
            set { _colorRetracementText[6]   = value; }
        }
		[Browsable(false)]
		public string RetracementLineLevel100TextSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorRetracementText[6]); }
			set { _colorRetracementText[6] = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        #endregion
				
		#region DashStyle
		[XmlIgnore()]
        [Description("Retracement Line 0% DashStyle.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 0% DashStyle")]
        public DashStyle RetracementLineLevel00DashStyle
        {
            get { return _dsRetracementLine[0]; }
            set { _dsRetracementLine[0]   = value; }
        }
		[XmlIgnore()]
        [Description("Retracement Line 23.6% DashStyle.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 23.6% DashStyle")]
        public DashStyle RetracementLineLevel23DashStyle
        {
            get { return _dsRetracementLine[1]; }
            set { _dsRetracementLine[1]   = value; }
        }
		[XmlIgnore()]
        [Description("Retracement Line 38.2% DashStyle.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 38.2% DashStyle")]
        public DashStyle RetracementLineLevel38DashStyle
        {
            get { return _dsRetracementLine[2]; }
            set { _dsRetracementLine[2]   = value; }
        }
		[XmlIgnore()]
        [Description("Retracement Line 50% DashStyle.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 50% DashStyle")]
        public DashStyle RetracementLineLevel50DashStyle
        {
            get { return _dsRetracementLine[3]; }
            set { _dsRetracementLine[3]   = value; }
        }
		[XmlIgnore()]
        [Description("Retracement Line 62.8% DashStyle.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 62.8% DashStyle")]
        public DashStyle RetracementLineLevel62DashStyle
        {
            get { return _dsRetracementLine[4]; }
            set { _dsRetracementLine[4]   = value; }
        }
		[XmlIgnore()]
        [Description("Retracement Line 76.4% DashStyle.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 76.4% DashStyle")]
        public DashStyle RetracementLineLevel76DashStyle
        {
            get { return _dsRetracementLine[5]; }
            set { _dsRetracementLine[5]   = value; }
        }
		[XmlIgnore()]
        [Description("Retracement Line 100% DashStyle.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 100% DashStyle")]
        public DashStyle RetracementLineLevel100DashStyle
        {
            get { return _dsRetracementLine[6]; }
            set { _dsRetracementLine[6]   = value; }
        }
        #endregion
	
		#region FontSize
        [Description("Line 0% Font Size.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 0% Font Size")]
        public int RetracementLineLevel100FontSize
        {
            get { return _intRetracementFontSize[0]; }
            set { _intRetracementFontSize[0] = Math.Max(0, value); }
        }
        [Description("Line 23.6% Font Size.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 23.6% Font Size")]
        public int RetracementLineLevel123FontSize
        {
            get { return _intRetracementFontSize[1]; }
            set { _intRetracementFontSize[1] = Math.Max(0, value); }
        }
        [Description("Line 38.2% Font Size.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 38.2% Font Size")]
        public int RetracementLineLevel138FontSize
        {
            get { return _intRetracementFontSize[2]; }
            set { _intRetracementFontSize[2]   = Math.Max(0, value); }
        }
        [Description("Line 50% Font Size.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 50% Font Size")]
        public int RetracementLineLevel150FontSize
        {
            get { return _intRetracementFontSize[3]; }
            set { _intRetracementFontSize[3] = Math.Max(0, value); }
        }
        [Description("Line 62.8% Font Size.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 62.8% Font Size")]
        public int RetracementLineLevel162FontSize
        {
            get { return _intRetracementFontSize[4]; }
            set { _intRetracementFontSize[4] = Math.Max(0, value); }
        }
        [Description("Line 76.4% Font Size.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 76.4% Font Size")]
        public int RetracementLineLevel176FontSize
        {
            get { return _intRetracementFontSize[5]; }
            set { _intRetracementFontSize[5] = Math.Max(0, value); }
        }
        [Description("Line 100% Font Size.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 100% Font Size")]
        public int RetracementLineLevel1100FontSize
        {
            get { return _intRetracementFontSize[6]; }
            set { _intRetracementFontSize[6] = Math.Max(0, value); }
        }
        #endregion
	
		#region FontSize
        [Description("Line 0% Width.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 0% Width")]
        public int RetracementLineLevel100Width
        {
            get { return _intRetracementLineWidth[0]; }
            set { _intRetracementLineWidth[0] = Math.Max(0, value); }
        }
        [Description("Line 23.6% Width.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 23.6% Width")]
        public int RetracementLineLevel123Width
        {
            get { return _intRetracementLineWidth[1]; }
            set { _intRetracementLineWidth[1] = Math.Max(0, value); }
        }
        [Description("Line 38.2% Width.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 38.2% Width")]
        public int RetracementLineLevel138Width
        {
            get { return _intRetracementLineWidth[2]; }
            set { _intRetracementLineWidth[2]   = Math.Max(0, value); }
        }
        [Description("Line 50% Width.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 50% Width")]
        public int RetracementLineLevel150Width
        {
            get { return _intRetracementLineWidth[3]; }
            set { _intRetracementLineWidth[3] = Math.Max(0, value); }
        }
        [Description("Line 62.8% Width.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 62.8% Width")]
        public int RetracementLineLevel162Width
        {
            get { return _intRetracementLineWidth[4]; }
            set { _intRetracementLineWidth[4] = Math.Max(0, value); }
        }
        [Description("Line 76.4% Width.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 76.4% Width")]
        public int RetracementLineLevel176Width
        {
            get { return _intRetracementLineWidth[5]; }
            set { _intRetracementLineWidth[5] = Math.Max(0, value); }
        }
        [Description("Line 100% Width.")]
        [Category("09-Wave 2 and 4 Retracement Lines")]
		[Gui.Design.DisplayName("Line 100% Width")]
        public int RetracementLineLevel1100Width
        {
            get { return _intRetracementLineWidth[6]; }
            set { _intRetracementLineWidth[6] = Math.Max(0, value); }
        }
        #endregion
        #endregion

        #region Projection Lines
        // Wave 3
        [Description("Projection Line ON.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Projection Line ON")]
        public bool PlotWave3PriceProjection
        {
            get { return this._boolPlotWave3PriceProjection; }
            set { this._boolPlotWave3PriceProjection = value; }
        }
        [Description("Line color.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Line Color")]
        public Color Wave3PriceProjectionLinesColor
        {
            get { return this._colorWave3PriceProjectionLines; }
            set { this._colorWave3PriceProjectionLines = value; }
        }
   		[Browsable(false)]
		public string Wave3PriceProjectionLinesColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWave3PriceProjectionLines); }
			set { _colorWave3PriceProjectionLines = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [Description("Text Color.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Text Color")]
        public Color Wave3PriceProjectionTexColor
        {
            get { return this._colorWave3PriceProjectionText; }
            set { this._colorWave3PriceProjectionText = value; }
        }
        [Browsable(false)]
        public string Wave3PriceProjectionTexColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWave3PriceProjectionText); }
            set { _colorWave3PriceProjectionText = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Description("Line DashStyle.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Line DashStyle")]
        public DashStyle Wave3PriceProjectionLinesDashStyle
        {
            get { return this._dsWave3PriceProjectionLines; }
            set { this._dsWave3PriceProjectionLines = value; }
        }
        [Description("Factor 1.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Factor 1")]
        public double Wave3PriceProjectionFactor1
        {
            get { return this._doubleWave3Factor1; }
            set { this._doubleWave3Factor1 = value; }
        }
        [Description("Factor 2.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Factor 2")]
        public double Wave3PriceProjectionFactor2
        {
            get { return this._doubleWave3Factor2; }
            set { this._doubleWave3Factor2 = value; }
        }
        [Description("Factor 3.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Factor 3")]
        public double Wave3PriceProjectionFactor3
        {
            get { return this._doubleWave3Factor3; }
            set { this._doubleWave3Factor3 = value; }
        }
        [Description("Text Font Size.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Text Font Size")]
        public int Wave3PriceProjectionTextFontSize
        {
            get { return _intWave3PriceProjectionFontSize; }
            set { _intWave3PriceProjectionFontSize = Math.Max(0, value); }
        }
        [Description("Line Width.")]
        [Category("10-Wave 3 Price Projection Lines")]
        [Gui.Design.DisplayName("Line Width")]
        public int Wave3PriceProjectionLinesWidth
        {
            get { return _intWave3PriceProjectionLinesWidth; }
            set { _intWave3PriceProjectionLinesWidth = Math.Max(0, value); }
        }
        // Wave 5
        [Description("Projection Line ON.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Projection Line ON")]
        public bool PlotWave5PriceProjection
        {
            get { return this._boolPlotWave5PriceProjection; }
            set { this._boolPlotWave5PriceProjection = value; }
        }
        [Description("Line color.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Line Color")]
        public Color Wave5PriceProjectionLinesColor
        {
            get { return this._colorWave5PriceProjectionLines; }
            set { this._colorWave5PriceProjectionLines = value; }
        }
   		[Browsable(false)]
		public string Wave5PriceProjectionLinesColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWave5PriceProjectionLines); }
			set { _colorWave5PriceProjectionLines = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [Description("Text Color.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Text Color")]
        public Color Wave5PriceProjectionTexColor
        {
            get { return this._colorWave5PriceProjectionText; }
            set { this._colorWave5PriceProjectionText = value; }
        }
        [Browsable(false)]
        public string Wave5PriceProjectionTexColorSerialize
        {
            get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorWave5PriceProjectionText); }
            set { _colorWave5PriceProjectionText = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
        }
        [Description("Line DashStyle.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Line DashStyle")]
        public DashStyle Wave5PriceProjectionLinesDashStyle
        {
            get { return this._dsWave5PriceProjectionLines; }
            set { this._dsWave5PriceProjectionLines = value; }
        }
        [Description("Factor 1.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Factor 1")]
        public double Wave5PriceProjectionFactor
        {
            get { return this._doubleWave5Factor; }
            set { this._doubleWave5Factor = value; }
        }
        [Description("Factor 2.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Factor 2")]
        public double Wave5PriceProjectionFactor2
        {
            get { return this._doubleWave5Factor2; }
            set { this._doubleWave5Factor2 = value; }
        }
        [Description("Factor 3.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Factor 3")]
        public double Wave5PriceProjectionFactor3
        {
            get { return this._doubleWave5Factor3; }
            set { this._doubleWave5Factor3 = value; }
        }
        [Description("Text Font Size.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Text Font Size")]
        public int Wave5PriceProjectionTextFontSize
        {
            get { return _intWave5PriceProjectionFontSize; }
            set { _intWave5PriceProjectionFontSize = Math.Max(0, value); }
        }
        [Description("Line Width.")]
        [Category("11-Wave 5 Price Projection Lines")]
        [Gui.Design.DisplayName("Line Width")]
        public int Wave5PriceProjectionLinesWidth
        {
            get { return _intWave5PriceProjectionLinesWidth; }
            set { _intWave5PriceProjectionLinesWidth = Math.Max(0, value); }
        }
        #endregion

		#region Elliott Oscillator Filter
        [Description("Turning this ON will only display waves in the direction of the trend.")]
        [Category("12-Elliot Oscillator Filter")]
		[Gui.Design.DisplayName("Filter ON")]
        public bool ElliottOscillatorFilterON
        {
            get { return this._boolElliottOscillatorFilterON; }
            set { this._boolElliottOscillatorFilterON = value; }
        }
		[Description("Increase Fast and Slow SMA length by this number.")]
        [Category("12-Elliot Oscillator Filter")]
		[Gui.Design.DisplayName("Multiplier")]
		public int ElliottOscillatorFilterMultiplier
		{
			get { return this._intElliottOscillatorFilterMultiplier; }
			set { _intElliottOscillatorFilterMultiplier = Math.Max(1, value); }
		}
		[Description("Fast SMA length.")]
        [Category("12-Elliot Oscillator Filter")]
		[Gui.Design.DisplayName("Fast SMA Length")]
		public int Fast
		{
			get { return _intFast; }
			set { _intFast = Math.Max(1, value); }
		}
		[Description("Minimun number of bars before trend considered established.")]
        [Category("12-Elliot Oscillator Filter")]
		[Gui.Design.DisplayName("Look Backs Bars")]
		public int LookBackBars
		{
			get { return _intLookBackBars; }
			set { _intLookBackBars = Math.Max(1, value); }
		}
		[Description("Slow SMA length.")]
        [Category("12-Elliot Oscillator Filter")]
		[Gui.Design.DisplayName("Slow SMA Length")]
		public int Slow
		{
			get { return _intSlow; }
			set { _intSlow = Math.Max(1, value); }
		}
		#endregion

		#region Linear Regression
        [Description("Turning this ON will display a linear regression for the current wave.")]
        [Category("13-Linear Regression")]
		[Gui.Design.DisplayName("Linear Regression ON")]
        public bool LinearRegressionON
        {
            get { return this._boolLinearRegressionON; }
            set { this._boolLinearRegressionON = value; }
        }
        [Description("Linear Regression lines color.")]
        [Category("13-Linear Regression")]
        [Gui.Design.DisplayName("Line Color")]
        public Color LinearRegressionLinesColor
        {
            get { return this._colorLinearRegressionLines; }
            set { this._colorLinearRegressionLines = value; }
        }
   		[Browsable(false)]
		public string LinearRegressionLinesColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorLinearRegressionLines); }
			set { _colorLinearRegressionLines = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [Description("Linear Regression background color.")]
        [Category("13-Linear Regression")]
        [Gui.Design.DisplayName("Background Color")]
        public Color LinearRegressionBackgroundColor
        {
            get { return this._colorLinearRegressionBackground; }
            set { this._colorLinearRegressionBackground = value; }
        }
   		[Browsable(false)]
		public string LinearRegressionBackgroundColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(_colorLinearRegressionBackground); }
			set { _colorLinearRegressionBackground = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
        [Description("Middle Line DashStyle.")]
        [Category("13-Linear Regression")]
        [Gui.Design.DisplayName("Middle Line DashStyle")]
        public DashStyle LinearRegressionMiddleLine
        {
            get { return this._dsLinearRegressionMiddleLine; }
            set { this._dsLinearRegressionMiddleLine = value; }
        }
        [Description("Channel Lines DashStyle.")]
        [Category("13-Linear Regression")]
        [Gui.Design.DisplayName("Channel Lines DashStyle")]
        public DashStyle LinearRegressionChannelLines
        {
            get { return this._dsLinearRegressionChannelLines; }
            set { this._dsLinearRegressionChannelLines = value; }
        }
        [Description("Standard Deviation.")]
        [Category("13-Linear Regression")]
        [Gui.Design.DisplayName("Standard Deviation")]
        public double StandardDeviation
        {
            get { return this._doubleStandardDeviation; }
            set { this._doubleStandardDeviation = value; }
        }
		[Description("Middle line width.")]
        [Category("13-Linear Regression")]
		[Gui.Design.DisplayName("Middle Line Width")]
		public int LinearRegressionMiddleLineWidth
		{
			get { return this._intLinearRegressionMiddleLineWidth; }
			set { _intLinearRegressionMiddleLineWidth = Math.Max(1, value); }
		}
		[Description("Channel lines width.")]
        [Category("13-Linear Regression")]
		[Gui.Design.DisplayName("Channel Lines Width")]
		public int LinearRegressionChannelLinesWidth
		{
			get { return this._intLinearRegressionChannelLinesWidth; }
			set { _intLinearRegressionChannelLinesWidth = Math.Max(1, value); }
		}
		#endregion
		
		#endregion
    }
}

public enum TDDirection
{
	LookForHighs,
	LookForLows
}

public enum TDWave
{
	WaveStart,
	Wave1,
	Wave2,
	Wave3,
	Wave4,
	Wave5,
	WaveA,
	WaveB,
	WaveC
}

public enum TDWaveType
{
	DownWave,
	UpWave
}

public struct TDWaveHelper 
{
	public int TDIndex;
	public string TDWaveTag;
	public int TDWaveBar;
	public double TDWaveHigh;
	public double TDWaveLow;
	public double TDWaveClose;
	public string TDWaveName;
	public Color TDWaveColor;
	
	public TDWaveHelper(int intIndex, string strTag, int intBar, double doubleHigh, double doubleLow, double doubleClose, string strName, Color colorWave) 
	{
		TDIndex = intIndex;
		TDWaveTag = strTag;
		TDWaveBar = intBar;    
		TDWaveHigh = doubleHigh;    
		TDWaveLow = doubleLow;    
		TDWaveClose = doubleClose;    
		TDWaveName = strName;    
		TDWaveColor = colorWave;    
	}
}

public struct TDWaveRequirements 
{
	public int TDWaveBars1;
	public int TDWaveBars2;
	public TDDirection TDSingleWaveDirection;
	public string TDWaveName;
	public bool TDPlotWave;
	public Color TDWaveColor;
	public DashStyle TDWaveDashStyle;
	public int TDWaveWidth;
	
	public TDWaveRequirements(int intWaveBars1, int intWaveBars2, TDDirection directionTDSingleWaveDirection, string strWaveName, bool boolPlotWave, Color colorTDWave, DashStyle dsTDWave, int intTDWaveWidth) 
	{
		TDWaveBars1 = intWaveBars1;
		TDWaveBars2 = intWaveBars2;
		TDSingleWaveDirection = directionTDSingleWaveDirection;    
		TDWaveName = strWaveName;    
		TDPlotWave = boolPlotWave;    
		TDWaveColor = colorTDWave;    
		TDWaveDashStyle = dsTDWave;    
		TDWaveWidth = intTDWaveWidth;    
	}
}

//public struct TDWaveRetracements 
//{
//	public double TDRetracementLevel;
//	public Color TDRetracementLineColor;
//	public Color TDRetracementTextColor;
//	public DashStyle TDRetracementDashStyle;
//	public FontStyle TDRetracementFontStyle;
//	public int TDRetracementFontSize;
//	public int TDRetracementWidth;
//	
//	public TDWaveRetracements(double doubleTDRetracementLevel, Color colorTDRetracementLine, Color colorTDRetracementText, DashStyle dsTDRetracement, FontStyle fsTDRetracement, int intTDRetracementFontSize, int intTDRetracementWidth) 
//	{
//		TDRetracementLevel = doubleTDRetracementLevel;
//		TDRetracementLineColor = colorTDRetracementLine;
//		TDRetracementTextColor = colorTDRetracementText;
//		TDRetracementDashStyle = dsTDRetracement;
//		TDRetracementFontStyle = fsTDRetracement;
//		TDRetracementFontSize = intTDRetracementFontSize;
//		TDRetracementWidth = intTDRetracementWidth;    
//	}
//}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private IGTDWaveUP[] cacheIGTDWaveUP = null;

        private static IGTDWaveUP checkIGTDWaveUP = new IGTDWaveUP();

        /// <summary>
        /// TD Wave UP
        /// </summary>
        /// <returns></returns>
        public IGTDWaveUP IGTDWaveUP()
        {
            return IGTDWaveUP(Input);
        }

        /// <summary>
        /// TD Wave UP
        /// </summary>
        /// <returns></returns>
        public IGTDWaveUP IGTDWaveUP(Data.IDataSeries input)
        {

            if (cacheIGTDWaveUP != null)
                for (int idx = 0; idx < cacheIGTDWaveUP.Length; idx++)
                    if (cacheIGTDWaveUP[idx].EqualsInput(input))
                        return cacheIGTDWaveUP[idx];

            IGTDWaveUP indicator = new IGTDWaveUP();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.SetUp();

            IGTDWaveUP[] tmp = new IGTDWaveUP[cacheIGTDWaveUP == null ? 1 : cacheIGTDWaveUP.Length + 1];
            if (cacheIGTDWaveUP != null)
                cacheIGTDWaveUP.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheIGTDWaveUP = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// TD Wave UP
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IGTDWaveUP IGTDWaveUP()
        {
            return _indicator.IGTDWaveUP(Input);
        }

        /// <summary>
        /// TD Wave UP
        /// </summary>
        /// <returns></returns>
        public Indicator.IGTDWaveUP IGTDWaveUP(Data.IDataSeries input)
        {
            return _indicator.IGTDWaveUP(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// TD Wave UP
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.IGTDWaveUP IGTDWaveUP()
        {
            return _indicator.IGTDWaveUP(Input);
        }

        /// <summary>
        /// TD Wave UP
        /// </summary>
        /// <returns></returns>
        public Indicator.IGTDWaveUP IGTDWaveUP(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.IGTDWaveUP(input);
        }

    }
}
#endregion
